import { useMutation } from '@apollo/client'
import React from 'react'
import { useTranslation } from 'react-i18next'

import { useAuth } from '@holi/core/auth/helpers/useAuth'
import { useErrorHandling } from '@holi/core/errors/hooks'
import { LinkType, useShareLink } from '@holi/core/hooks/useShareLink'
import useRouting from '@holi/core/navigation/hooks/useRouting'
import { FavoriteType } from '@holi/core/screens/favorites/model'
import {
  FavoriteResponse,
  UnfavoriteResponse,
  favoriteMutation,
  unfavoriteMutation,
} from '@holi/core/screens/favorites/mutation'
import { favoritesQuery } from '@holi/core/screens/favorites/queries'
import { insightById } from '@holi/core/screens/insights/queries'
import { Insight } from '@holi/core/screens/insights/types'
import { Bookmark3Line, BookmarkFill, BookmarkLine, Share } from '@holi/icons/src/generated'
import { HoliToastType, useToast } from '@holi/ui/components/molecules/HoliToastProvider'
import HoliActionDrawer from '@holi/ui/components/organisms/HoliActionDrawer'

type InsightDrawerProps = {
  insight: Insight
  children: React.ReactNode
}

const InsightDrawer: React.FC<InsightDrawerProps> = ({ insight, children }: InsightDrawerProps) => {
  const { t } = useTranslation()
  const { navigate } = useRouting()
  const { isLoggedIn } = useAuth()
  const { displayError } = useErrorHandling()
  const { openToast } = useToast()

  const { id, isFavorite } = insight

  const [favorite] = useMutation<FavoriteResponse>(favoriteMutation, {
    onError: (error) =>
      displayError(error, 'Failed to favorite insight', {
        location: 'InsightDrawer.favoriteInsight',
        id,
      }),
    refetchQueries: [{ query: insightById, variables: { id } }, favoritesQuery],
  })

  const [unfavorite] = useMutation<UnfavoriteResponse>(unfavoriteMutation, {
    onError: (error) =>
      displayError(error, 'Failed to remove insight favorite', {
        location: 'InsightDrawer.unfavorite',
        id,
      }),
    refetchQueries: [{ query: insightById, variables: { id } }, favoritesQuery],
  })

  const favoriteInsightActionHandler = async () => {
    if (isLoggedIn) {
      await favorite({ variables: { favoritableId: id, favoritableType: FavoriteType.INSIGHT } })
      openToast(t('insights.insight.favorite.success'), HoliToastType.SUCCESS)
    }
    navigate('/login')
  }

  const unfavoriteActionHandler = async () => {
    await unfavorite({ variables: { favoritableId: id, favoritableType: FavoriteType.INSIGHT } })
    openToast(t('insights.insight.remove.favorite.success'), HoliToastType.SUCCESS)
  }

  const seeAllFavoritesActionHandler = async () => {
    navigate('/profile/settings/favorites')
  }

  const [, shareLink] = useShareLink({
    link: `/insights/${insight.id}`,
    linkType: LinkType.UNIVERSAL,
    message: `${t('insights.insight.share.message', {
      title: insight.title,
      interpolation: { escapeValue: false },
    })}`,
  })

  return (
    <HoliActionDrawer.Drawer label={t('global.moreOptions')} testID="btn-insight-more-options" buttonElement={children}>
      <HoliActionDrawer.Action
        icon={Share}
        title={t('global.share')}
        testID="btn-share-insight"
        handleAction={shareLink}
      />
      {isFavorite ? (
        <HoliActionDrawer.Action
          icon={BookmarkFill}
          title={t('insights.insight.remove.favorite.button')}
          testID="btn-remove-insight-favorite"
          handleAction={unfavoriteActionHandler}
        />
      ) : (
        <HoliActionDrawer.Action
          icon={BookmarkLine}
          title={t('insights.insight.favorite.button')}
          testID="btn-favorite-insight"
          handleAction={favoriteInsightActionHandler}
        />
      )}
      {isLoggedIn && (
        <HoliActionDrawer.Action
          icon={Bookmark3Line}
          title={t('favorites.seeAll.button')}
          testID="btn-see-all-favorites"
          handleAction={seeAllFavoritesActionHandler}
        />
      )}
    </HoliActionDrawer.Drawer>
  )
}

export default InsightDrawer
